﻿using System;
using System.Collections.Generic;

namespace UMK.WzorceOperacyjne
{
    //https://sourcemaking.com/design_patterns/observer/cpp/1

    class Dzielenie : IObserver<int> //obserwator
    {
        private int wartość = 0;
        private int dzielnik;

        public Dzielenie(int dzielnik)
        {
            this.dzielnik = dzielnik;
        }

        public void OnNext(int dzielna)
        //public void Aktualizuj(int dzielna)
        {
            wartość = dzielna / dzielnik;
            Console.WriteLine("{0} / {1} = {2}", dzielna, dzielnik, wartość);
        }

        public void OnCompleted()
        {
            Console.WriteLine("Koniec z dzieleniem");
        }

        public void OnError(Exception error)
        {
            Console.WriteLine("Błąd w dzieleniu: " + error.Message);
        }
    }

    class Modulo : IObserver<int> //obserwator
    {
        private int wartość = 0;
        private int dzielnik;

        public Modulo(int dzielnik)
        {
            this.dzielnik = dzielnik;
        }

        public void OnNext(int dzielna)
        {
            wartość = dzielna % dzielnik;
            Console.WriteLine("{0} % {1} = {2}", dzielna, dzielnik, wartość);
        }

        public void OnCompleted()
        {
            Console.WriteLine("Koniec z modulo");
        }

        public void OnError(Exception error)
        {
            Console.WriteLine("Błąd w modulo: " + error.Message);
        }
    }

    class Podmiot : IObservable<int> //podmiot, obserwabla
    {
        private int wartość;
        private List<IObserver<int>> obserwatorzy = new List<IObserver<int>>();

        private void powiadom()
        {
            foreach (IObserver<int> obserwator in obserwatorzy)
                obserwator.OnNext(wartość);
        }

        public Podmiot()
        {
            wartość = 0;
        }

        class Unsubscriber : IDisposable
        {
            private Podmiot podmiot = null;
            private IObserver<int> obserwator;

            public Unsubscriber(Podmiot podmiot, IObserver<int> obserwator)
            {
                this.podmiot = podmiot;
                this.obserwator = obserwator;
            }

            public void Dispose()
            {
                podmiot.odłączObserwatora(obserwator);
                obserwator.OnCompleted();
            }
        }

        public IDisposable Subscribe(IObserver<int> obserwator)
        {
            if (!obserwatorzy.Contains(obserwator))
            {
                obserwatorzy.Add(obserwator);
                return new Unsubscriber(this, obserwator);
            }
            else return null;
        }
        private void odłączObserwatora(IObserver<int> obserwator)
        {
            if (obserwatorzy.Contains(obserwator))
                obserwatorzy.Remove(obserwator);
        }

        public void ZmieńWartość(int nowaWartość)
        {
            wartość = nowaWartość;
            powiadom();
        }
    }
}